#include "GUI/Model/Data/ProjectionItems.h"
#include "GUI/Model/Device/MaskItems.h"
#include "GUI/Model/Model/JobModel.h"
#include "Tests/GTestWrapper/google_test.h"
#include <memory>

//! Test Widget which logs calling activity of ModelMapper
class Widget {
public:
    Widget()
        : m_onPropertyChangeCount(0)
        , m_onChildPropertyChangeCount(0)
        , m_onChildrenChangeCount(0)
        , m_onAboutToRemoveChild(0)
    {
    }

    void clear()
    {
        m_onPropertyChangeCount = 0;
        m_onChildPropertyChangeCount = 0;
        m_onChildrenChangeCount = 0;
        m_onAboutToRemoveChild = 0;
        m_reported_items.clear();
        m_reported_names.clear();
    }

    void subscribe(ModelMapper* mapper, bool with_subscription = false)
    {
        clear();

        void* caller = (with_subscription ? this : nullptr);

        mapper->setOnPropertyChange([this](QString name) { onPropertyChange(name); }, caller);

        mapper->setOnChildPropertyChange(
            [this](SessionItem* item, QString name) { onChildPropertyChange(item, name); }, caller);

        mapper->setOnChildrenChange([this](SessionItem*) { onChildrenChange(); }, caller);

        mapper->setOnAboutToRemoveChild([this](SessionItem* item) { onAboutToRemoveChild(item); },
                                        caller);
    }

    void onPropertyChange(const QString& name)
    {
        m_reported_names.append(name);
        m_onPropertyChangeCount++;
    }

    void onChildPropertyChange(SessionItem* item, const QString& name)
    {
        m_reported_items.append(item);
        m_reported_names.append(name);
        m_onChildPropertyChangeCount++;
    }

    void onChildrenChange() { m_onChildrenChangeCount++; }

    void unsubscribe(ModelMapper* mapper) { mapper->unsubscribe(this); }

    void onAboutToRemoveChild(SessionItem* item)
    {
        m_reported_items.append(item);
        m_onAboutToRemoveChild++;
    }

    int m_onPropertyChangeCount;
    int m_onChildPropertyChangeCount;
    int m_onChildrenChangeCount;
    int m_onAboutToRemoveChild;
    QList<SessionItem*> m_reported_items;
    QStringList m_reported_names;
};

class TestMapperForItem : public ::testing::Test {
public:
    TestMapperForItem()
        : m_mapped_item(nullptr)
    {
    }

    void setItem(SessionItem* item, Widget* widget = nullptr, bool with_subscription = false)
    {
        m_mapped_item = item;
        m_mapper = std::make_unique<ModelMapper>();
        m_mapper->setItem(item);
        if (widget)
            widget->subscribe(m_mapper.get(), with_subscription);
    }

    SessionItem* m_mapped_item;
    std::unique_ptr<ModelMapper> m_mapper;
};

TEST_F(TestMapperForItem, initialCondition)
{
    Widget w;
    EXPECT_EQ(w.m_onPropertyChangeCount, 0);
    EXPECT_EQ(w.m_onChildPropertyChangeCount, 0);
    EXPECT_EQ(w.m_onChildrenChangeCount, 0);
    EXPECT_TRUE(w.m_reported_items.isEmpty());
    EXPECT_TRUE(w.m_reported_names.isEmpty());
    EXPECT_TRUE(m_mapped_item == nullptr);
    EXPECT_TRUE(!m_mapper);
}
