#include "GUI/Model/BaseItem/PropertyItem.h"
#include "GUI/Model/Data/DataItem.h"
#include "GUI/Model/Data/RealItem.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Model/Device/InstrumentModel.h"
#include "GUI/Model/Device/MaskItems.h"
#include "GUI/Model/Job/JobItem.h"
#include "GUI/Model/Model/JobModel.h"
#include "GUI/Model/Sample/LayerItem.h"
#include "GUI/Model/Sample/SampleItem.h"
#include "GUI/Support/Data/SessionItemTags.h"
#include "Tests/GTestWrapper/google_test.h"
#include <QSignalSpy>
#include <QXmlStreamWriter>
#include <memory>

//! Testing SessionModel::setData notifications.

TEST(TestSessionModel, setData)
{
    SessionModel model("TestModel");
    auto* item = model.insertItem<PropertyItem>();

    QSignalSpy spy(&model, &SessionModel::dataChanged);

    QVariant data(42.0);

    // setting the data and checking that signal was emmitted only once
    EXPECT_TRUE(model.setData(model.indexOfItem(item), data, Qt::DisplayRole));
    EXPECT_EQ(spy.count(), 1);
    EXPECT_EQ(item->value().toDouble(), 42.0);
    QList<QVariant> arguments = spy.takeFirst();
    EXPECT_EQ(arguments.at(0).toModelIndex(), model.indexOfItem(item));

    // setting the same data second time
    EXPECT_FALSE(model.setData(model.indexOfItem(item), data, Qt::DisplayRole));
    EXPECT_EQ(spy.count(), 0);

    // setting another data
    EXPECT_TRUE(model.setData(model.indexOfItem(item), QVariant(43.0), Qt::DisplayRole));
    EXPECT_EQ(spy.count(), 1);
}

//! Test if SessionItem can be copied from one model to another. Particularly, we test
//! here if a SampleItem can be copied from SampleModel to the JobItem of JobModel

TEST(TestSessionModel, copyItem)
{
    auto sample1 = std::make_unique<SampleItem>();
    sample1->setSampleName("sample1");
    sample1->addStandardMaterials();

    InstrumentModel instrumentItems;
    auto* instrument1 = instrumentItems.addInstrumentItem<GISASInstrumentItem>();
    instrument1->setInstrumentName("instrument1");

    JobModel jobModel;
    auto* jobItem = jobModel.insertItem<JobItem>();

    jobItem->copySampleIntoJob(sample1.get());
    EXPECT_NE(jobItem->sampleItem(), nullptr);

    jobItem->copyInstrumentIntoJob(instrument1);
    EXPECT_NE(jobItem->instrumentItem(), nullptr);
}
